"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDebuggerAPI = void 0;
const extend_context_1 = require("./extend-context");
function createDebuggerAPI(bindings, apiNamespace, logger) {
    const extensionMap = new Map();
    let api;
    const resultPromise = new Promise((resolve, reject) => {
        api = { resumeExecution: resolve, failExecution: reject };
    });
    for (const key in bindings) {
        if (!bindings.hasOwnProperty(key)) {
            continue;
        }
        const getValue = bindings[key];
        try {
            getValue();
        }
        catch (error) {
            if (error instanceof ReferenceError) {
                // Variables can be declared after `await "debugger"` statement.
                continue;
            }
            throw error;
        }
        extensionMap.set(key, getValue);
    }
    extensionMap.set(apiNamespace, () => api);
    logger.info(`Async function is paused for debugging. The following variables ` +
        `are available: [${[...extensionMap.keys()].sort().join(', ')}].`);
    const applyToContexts = (contexts) => {
        const contextCleanups = contexts.map((context) => (extend_context_1.extendContext(context, extensionMap, logger)));
        return () => {
            logger.info('Async function execution is resumed.');
            for (const cleanup of contextCleanups) {
                cleanup();
            }
        };
    };
    return { applyToContexts, resultPromise, api };
}
exports.createDebuggerAPI = createDebuggerAPI;
