"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDebugAsyncBrowser = exports.debugAsync = void 0;
const create_debugger_api_1 = require("./create-debugger-api");
exports.debugAsync = createDebugAsyncBrowser();
function createDebugAsyncBrowser({ defaults: { contexts: contextsDefault = [globalThis], logger: defaultLogger = {}, apiNamespace: apiNamespaceDefault = 'AsyncDebugger' } = {} } = {}) {
    let isBeingDebugged = false;
    return async (bindings, options = {}) => {
        if (isBeingDebugged) {
            // Better to just ignore.
            return;
        }
        const { contexts = contextsDefault, apiNamespace = apiNamespaceDefault, logger: { info = console.info, warn = console.warn } = defaultLogger } = options;
        const logger = { info, warn };
        const { resultPromise, applyToContexts } = (create_debugger_api_1.createDebuggerAPI(bindings, apiNamespace, logger));
        const teardown = applyToContexts(contexts);
        try {
            isBeingDebugged = true;
            return await resultPromise;
        }
        finally {
            isBeingDebugged = false;
            teardown();
        }
    };
}
exports.createDebugAsyncBrowser = createDebugAsyncBrowser;
