"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDebugAsyncNode = exports.debugAsync = void 0;
const repl_1 = require("repl");
const create_debugger_api_1 = require("./create-debugger-api");
exports.debugAsync = createDebugAsyncNode();
function createDebugAsyncNode({ defaults: { contexts: contextsDefault = [globalThis], logger: defaultLogger = {}, apiNamespace: apiNamespaceDefault = 'AsyncDebugger', replOptions: replOptionsDefault } = {} } = {}) {
    let isBeingDebugged = false;
    return async (bindings, options = {}) => {
        if (isBeingDebugged) {
            // Better to just ignore.
            return;
        }
        const { contexts = contextsDefault, apiNamespace = apiNamespaceDefault, logger: { info = console.info, warn = console.warn } = defaultLogger, replOptions = replOptionsDefault } = options;
        const logger = { info, warn };
        const { resultPromise, applyToContexts, api } = (create_debugger_api_1.createDebuggerAPI(bindings, apiNamespace, logger));
        const repl = repl_1.start(replOptions);
        const exitPromise = new Promise((resolve) => {
            repl.once('exit', () => {
                api.resumeExecution();
                resolve();
            });
        });
        const teardown = applyToContexts([...contexts, repl.context]);
        try {
            isBeingDebugged = true;
            const result = await resultPromise;
            return result;
        }
        finally {
            teardown();
            repl.close();
            await exitPromise;
            isBeingDebugged = false;
        }
    };
}
exports.createDebugAsyncNode = createDebugAsyncNode;
